function esn = generate_esn(Nu, Nx, Ny)

%% Gnrer un ESN sous la forme d'une structure comprenant les paramtres
% les paramtres de taille Nx, Nu, Ny
% les matrices de paramtres Win, W, Wfb, Wout
% un paramtre de fuite : leakage
% un paramtre de rgularisation : lambda

% Paramtres de taille
esn.Nx = Nx; 
esn.Nu = Nu; 
esn.Ny = Ny; 
  
% Pour gnrer W, utiliser la densit suivante :
connectivity = min([10/Nx 1]);


success = 0 ;                                               
while success == 0
    % following block might fail, thus we repeat until we obtain a valid
    % W matrix
    try
        esn.W = sprand(Nx, Nx, connectivity);
        esn.W(esn.W ~= 0) = 2*(esn.W(esn.W ~= 0)  - 0.5);
        maxVal = 2*max(abs(eigs(esn.W,1)));
        esn.W = esn.W/maxVal;
        success = 1 ;
    catch
        success = 0 ; 
    end
end

% input weight matrix
esn.Win = 2.0 * randi(2,Nx, Nu)- 3.0; % values +1,-1

% output weight matrix
esn.Wout = zeros(Ny, Nx + Nu);

% output feedback weight matrix has weights in columns
esn.Wfb = zeros(Nx, Ny); % no feedback

esn.leakage = 1.0; % no leakage
esn.lambda = 0.0 ; % no regularization 

end
