clear all; 
close all;

%% Gnrer les donnes d'apprentissage et de test

nmax = 300;
P = 10*pi; % input period length

%%%% slow sine input 
arg1 = rand;
uapp = [ones(nmax,1),  sin( arg1 + 2 * pi * (1:nmax)' / P)];
arg2 = rand+2;
u = [ones(nmax,1),  sin( arg2 + 2 * pi * (1:nmax)' / P)];

%%%% output sequence : 1/2 * u(n)^7
yapp = 1/2 * uapp(:,2).^7;
y = 1/2 * u(:,2).^7;

figure(4); clf;
plot(uapp(:,2));
figure(5); clf;
plot(yapp(:,1));

%%%% Quelques paramtres intressants
Nu = 2; 
Nx = 100; 
Ny = 1; 
nmin = 100 ; % attendre l'tat d'cho
nPoints = nmax - nmin; % nombre de points restants


%% 1. Gnrer l'ESN

esn = generate_esn(Nu, Nx, Ny);

% Vous devrez ajuster ici quelques paramtres de l'ESN
% tels que esn.Wfb...
esn.Win = 2.0 * randi(2,Nx, Nu)- 3.0; % values +1,-1
esn.Wfb = zeros(Nx, Ny); % no feedback
% esn.Wfb = 1.0 .*(2.0 .* rand(Nx, Ny)- 1.0);


%% 2. Entraner l'ESN

[trainedEsn X] = train_esn(uapp, yapp, esn, nmin) ; 

% on trace l'volution de quatre tats
plot_states(X, [2 3 4 5], nPoints, 1) ; 

% Il vous reste  calculer ici le signal de sortie ytrain appris  partir de X

ytrain = tanh(X*(trainedEsn.Wout)'); 

% puis  calculer l'erreur d'apprentissage

trainError = sum((atanh(yapp(nmin:end,:)) - atanh(ytrain)).^2)/(2*nPoints);
fprintf('train Error = %d\n', trainError);

% on trace sur le mme graphe le signal yapp et le signal appris ytrain
plot_sequence(yapp(nmin:end,:), ytrain, nPoints,...
    'training: teacher sequence (red) vs predicted sequence (blue)');


%% 3. Tester l'ESN

ytest = test_esn(u, trainedEsn, nmin);

% Il vous reste  calculer l'erreur de test

testError = sum((atanh(y(nmin:end,:)) - ytest).^2)/(2*nPoints);
fprintf('test Error = %d\n', testError);

% on trace sur le mme graphe le signal y et le signal appris ytest
plot_sequence(y(nmin:end,:), ytest, nPoints, ...
    'testing: teacher sequence (red) vs predicted sequence (blue)') ;


%% 4. Observer la valeur moyenne des poids de Wout

fprintf('average absolute output weights: %g\n', mean(abs(trainedEsn.Wout)));