function [J grad] = nnCostFunction(nn_params, ...
                                   input_layer_size, ...
                                   hidden_layer_size, ...
                                   num_labels, ...
                                   X, y, lambda)
%NNCOSTFUNCTION Implements the neural network cost function for a two layer
%neural network which performs classification
%   [J grad] = NNCOSTFUNCTON(nn_params, hidden_layer_size, num_labels, ...
%   X, y, lambda) computes the cost and gradient of the neural network. The
%   parameters for the neural network are "unrolled" into the vector
%   nn_params and need to be converted back into the weight matrices. 
% 
%   The returned parameter grad should be a "unrolled" vector of the
%   partial derivatives of the neural network.
%

% Reshape nn_params back into the parameters Theta1 and Theta2, the weight matrices
% for our 2 layer neural network
Theta1 = reshape(nn_params(1:hidden_layer_size * (input_layer_size + 1)), ...
                 hidden_layer_size, (input_layer_size + 1));

Theta2 = reshape(nn_params((1 + (hidden_layer_size * (input_layer_size + 1))):end), ...
                 num_labels, (hidden_layer_size + 1));

% Setup some useful variables
N = size(X, 1);
         
% You need to return the following variables correctly 
J = 0;
Theta1_grad = zeros(size(Theta1));
Theta2_grad = zeros(size(Theta2));

% ====================== YOUR CODE HERE ======================

a1 = [ones(N, 1) X];
z2 = a1*Theta1';
a2 = [ones(size(z2, 1), 1) sigmoid(z2)];
z3 = a2*Theta2';
a3 = sigmoid(z3);

t = zeros(N, size(z3, 2));
for i = 1:N
t(i, y(i)) = 1;
endfor

J = -1/N*sum(sum(t .* log(a3) +(1-t) .* log(1-a3))) + \
    lambda/2*(sum(sum(Theta1(:, 2:end).^2))+sum(sum(Theta2(:, 2:end).^2)));
d3 = a3 - t ;
d2 = d3 * Theta2(:, 2:end) .* sigmoidGradient(z2);
Delta2 = d3' * a2;
Delta1 = d2' * a1;
Theta1_grad = Delta1/N + lambda .* [zeros(hidden_layer_size, 1) Theta1(:, 2:end)];
Theta2_grad = Delta2/N + lambda .* [zeros(num_labels, 1) Theta2(:, 2:end)];

% ============================================================

% Unroll gradients
grad = [Theta1_grad(:) ; Theta2_grad(:)];


end
